"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.authorizationSaga = void 0;
const eth_1 = require("web3x-es/eth");
const address_1 = require("web3x-es/address");
const effects_1 = require("redux-saga/effects");
const schemas_1 = require("@dcl/schemas");
const decentraland_transactions_1 = require("decentraland-transactions");
const eth_2 = require("../../lib/eth");
const chainConfiguration_1 = require("../../lib/chainConfiguration");
const ERC20_1 = require("../../contracts/ERC20");
const ERC721_1 = require("../../contracts/ERC721");
const utils_1 = require("./utils");
const actions_1 = require("./actions");
const types_1 = require("./types");
function* authorizationSaga() {
    yield effects_1.takeEvery(actions_1.FETCH_AUTHORIZATIONS_REQUEST, handleFetchAuthorizationsRequest);
    yield effects_1.takeEvery(actions_1.GRANT_TOKEN_REQUEST, handleGrantTokenRequest);
    yield effects_1.takeEvery(actions_1.REVOKE_TOKEN_REQUEST, handleRevokeTokenRequest);
}
exports.authorizationSaga = authorizationSaga;
function* handleFetchAuthorizationsRequest(action) {
    try {
        const { authorizations } = action.payload;
        const authorizationsToStore = [];
        for (const authorization of authorizations) {
            if (!utils_1.isValidType(authorization.type)) {
                throw new Error(`Invalid authorization type ${authorization.type}`);
            }
            const { chainId } = authorization;
            const address = address_1.Address.fromString(authorization.address);
            const tokenAddress = address_1.Address.fromString(authorization.tokenAddress);
            const authorizedAddress = address_1.Address.fromString(authorization.authorizedAddress);
            const provider = yield effects_1.call(() => eth_2.getNetworkProvider(chainId));
            const eth = new eth_1.Eth(provider);
            switch (authorization.type) {
                case types_1.AuthorizationType.ALLOWANCE:
                    const allowance = yield effects_1.call(() => new ERC20_1.ERC20(eth, tokenAddress).methods
                        .allowance(address, authorizedAddress)
                        .call());
                    if (parseInt(allowance, 10) > 0) {
                        authorizationsToStore.push(authorization);
                    }
                    break;
                case types_1.AuthorizationType.APPROVAL:
                    const isApproved = yield effects_1.call(() => new ERC721_1.ERC721(eth, tokenAddress).methods
                        .isApprovedForAll(address, authorizedAddress)
                        .call());
                    if (isApproved) {
                        authorizationsToStore.push(authorization);
                    }
                    break;
            }
        }
        yield effects_1.put(actions_1.fetchAuthorizationsSuccess(authorizationsToStore));
    }
    catch (error) {
        yield effects_1.put(actions_1.fetchAuthorizationsFailure(error.message));
    }
}
function* handleGrantTokenRequest(action) {
    try {
        const { authorization } = action.payload;
        const txHash = yield effects_1.call(() => changeAuthorization(authorization, types_1.AuthorizationAction.GRANT));
        yield effects_1.put(actions_1.grantTokenSuccess(authorization, authorization.chainId, txHash));
    }
    catch (error) {
        yield effects_1.put(actions_1.grantTokenFailure(error.message));
    }
}
function* handleRevokeTokenRequest(action) {
    try {
        const { authorization } = action.payload;
        const txHash = yield effects_1.call(() => changeAuthorization(authorization, types_1.AuthorizationAction.REVOKE));
        yield effects_1.put(actions_1.revokeTokenSuccess(authorization, authorization.chainId, txHash));
    }
    catch (error) {
        yield effects_1.put(actions_1.revokeTokenFailure(error.message));
    }
}
function changeAuthorization(authorization, action) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!utils_1.isValidType(authorization.type)) {
            throw new Error(`Invalid authorization type ${authorization.type}`);
        }
        const provider = yield eth_2.getConnectedProvider();
        if (!provider) {
            throw new Error('Could not connect to Ethereum');
        }
        const eth = new eth_1.Eth(provider);
        const { network } = chainConfiguration_1.getChainConfiguration(authorization.chainId);
        const from = address_1.Address.fromString(authorization.address);
        const tokenAddress = address_1.Address.fromString(authorization.tokenAddress);
        const authorizedAddress = address_1.Address.fromString(authorization.authorizedAddress);
        const chainId = authorization.chainId;
        let method;
        switch (authorization.type) {
            case types_1.AuthorizationType.ALLOWANCE:
                const amount = action === types_1.AuthorizationAction.GRANT ? utils_1.getTokenAmountToApprove() : 0;
                method = new ERC20_1.ERC20(eth, tokenAddress).methods.approve(authorizedAddress, amount);
                break;
            case types_1.AuthorizationType.APPROVAL:
                const isApproved = action === types_1.AuthorizationAction.GRANT;
                method = new ERC721_1.ERC721(eth, tokenAddress).methods.setApprovalForAll(authorizedAddress, isApproved);
                break;
        }
        switch (network) {
            case schemas_1.Network.ETHEREUM:
                return method.send({ from }).getTxHash();
            case schemas_1.Network.MATIC:
                const payload = method.getSendRequestPayload({ from });
                const txData = payload.params[0].data;
                const metaTxProvider = yield eth_2.getNetworkProvider(chainId);
                return decentraland_transactions_1.sendMetaTransaction(provider, metaTxProvider, txData, decentraland_transactions_1.getContract(decentraland_transactions_1.getContractName(tokenAddress.toString()), chainId));
        }
    });
}
//# sourceMappingURL=sagas.js.map