"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const chai_1 = require("chai");
const schemas_1 = require("@dcl/schemas");
const utils_1 = require("./utils");
describe('modules', function () {
    describe('transaction', function () {
        describe('utils', function () {
            const hash = '0xdeadbeef';
            const payload = { some: 'data' };
            const chainId = schemas_1.ChainId.ETHEREUM_MAINNET;
            const tx = { hash, payload };
            const txWithReceipt = { hash, payload, withReceipt: true };
            describe('buildTransactionPayload', function () {
                it('should return a new object with the transaction flag an the action inside', function () {
                    const txPayload = utils_1.buildTransactionPayload(chainId, tx.hash, tx.payload);
                    chai_1.expect(txPayload).to.deep.equal({
                        [utils_1.TRANSACTION_ACTION_FLAG]: tx
                    });
                });
                it('should support only supplying the transaction hash', function () {
                    const tx = { hash };
                    const txPayload = utils_1.buildTransactionPayload(chainId, tx.hash);
                    chai_1.expect(txPayload).to.deep.equal({
                        [utils_1.TRANSACTION_ACTION_FLAG]: Object.assign(Object.assign({}, tx), { payload: {} })
                    });
                });
            });
            describe('buildTransactionWithReceiptPayload', function () {
                it('should return a new object with the transaction flag an the action inside', function () {
                    const txPayload = utils_1.buildTransactionWithReceiptPayload(chainId, tx.hash, tx.payload);
                    chai_1.expect(txPayload).to.deep.equal({
                        [utils_1.TRANSACTION_ACTION_FLAG]: txWithReceipt
                    });
                });
                it('should support only supplying the transaction hash', function () {
                    const tx = { hash };
                    const txPayload = utils_1.buildTransactionWithReceiptPayload(chainId, tx.hash);
                    chai_1.expect(txPayload).to.deep.equal({
                        [utils_1.TRANSACTION_ACTION_FLAG]: Object.assign(Object.assign({}, txWithReceipt), { payload: {} })
                    });
                });
            });
            describe('isTransactionAction', function () {
                it('should return true if the action was built with buildTransactionPayload', function () {
                    const txPayload = utils_1.buildTransactionPayload(chainId, tx.hash, tx.payload);
                    const action = {
                        type: '[Success] Transaction action',
                        payload: txPayload
                    };
                    chai_1.expect(utils_1.isTransactionAction(action)).to.equal(true);
                });
                it('should return true if the action was built with buildTransactionWithReceiptPayload', function () {
                    const txPayload = utils_1.buildTransactionWithReceiptPayload(chainId, tx.hash, tx.payload);
                    const action = {
                        type: '[Success] Transaction action',
                        payload: txPayload
                    };
                    chai_1.expect(utils_1.isTransactionAction(action)).to.equal(true);
                });
                it('should return false for normal actions', function () {
                    chai_1.expect(utils_1.isTransactionAction({
                        type: '[Success] Some success action'
                    })).to.equal(false);
                    chai_1.expect(utils_1.isTransactionAction({
                        type: '[Request] Some request action',
                        payload: {
                            mock_transaction_flag: tx
                        }
                    })).to.equal(false);
                });
            });
            describe('getTransactionFromAction', function () {
                it('should return the transaction from a built transaction action with buildTransactionPayload', function () {
                    const txPayload = utils_1.buildTransactionWithReceiptPayload(chainId, tx.hash, tx.payload);
                    const action = {
                        type: '[Success] Transaction action',
                        payload: Object.assign({ this: 'is', more: 2, data: ['a', 3] }, txPayload)
                    };
                    chai_1.expect(utils_1.getTransactionFromAction(action)).to.deep.equal(txWithReceipt);
                });
                it('should return the transaction from a built transaction action with buildTransactionWithReceiptPayload', function () {
                    const txPayload = utils_1.buildTransactionPayload(chainId, tx.hash, tx.payload);
                    const action = {
                        type: '[Success] Transaction action',
                        payload: Object.assign({ this: 'is', more: 2, data: ['a', 3] }, txPayload)
                    };
                    chai_1.expect(utils_1.getTransactionFromAction(action)).to.deep.equal(tx);
                });
                it('should return undefined for a normal action', function () {
                    const action = {
                        type: '[Success] Transaction action',
                        payload: { this: 'is', more: 2, data: ['a', 3] }
                    };
                    chai_1.expect(utils_1.getTransactionFromAction(action)).to.equal(undefined);
                });
            });
            describe('getTransactionHashFromAction', function () {
                it('should return the transaction hash from a built transaction action with buildTransactionPayload', function () {
                    const txPayload = utils_1.buildTransactionPayload(chainId, tx.hash, tx.payload);
                    const action = {
                        type: '[Success] Transaction action',
                        payload: Object.assign({ data: ['a', 3] }, txPayload)
                    };
                    chai_1.expect(utils_1.getTransactionHashFromAction(action)).to.deep.equal(tx.hash);
                });
                it('should return the transaction hash from a built transaction action with buildTransactionWithReceiptPayload', function () {
                    const txPayload = utils_1.buildTransactionWithReceiptPayload(chainId, tx.hash, tx.payload);
                    const action = {
                        type: '[Success] Transaction action',
                        payload: Object.assign({ data: ['a', 3] }, txPayload)
                    };
                    chai_1.expect(utils_1.getTransactionHashFromAction(action)).to.deep.equal(tx.hash);
                });
                it('should throw for a normal action (use isTransactionAction to avoid it)', function () {
                    const action = {
                        type: '[Success] Transaction action',
                        payload: { more: 2 }
                    };
                    chai_1.expect(() => utils_1.getTransactionHashFromAction(action)).to.throw("Cannot read property 'hash' of undefined");
                });
            });
        });
    });
});
//# sourceMappingURL=utils.spec.js.map